@@ -44,6 +44,7 @@ class Ensemble:
         self.total_best_sum = None  # total sum of predictions, the oof of ensemble
         self.target = None
         self.target_columns = None
+        self.sample_weight = None
         self._ml_task = ml_task
         self._optimize_metric = optimize_metric
         self._is_stacked = is_stacked
@@ -80,7 +81,7 @@ class Ensemble:
         """ Needed when ensemble is treated as model and we want to compute additional metrics for it """
         # single prediction (in case of binary classification and regression)
         if self.oof_predictions is not None:
-            return self.oof_predictions
+            return self.oof_predictions.copy(deep=True)
 
         if self.total_best_sum.shape[1] == 1:
             tmp_df = pd.DataFrame({"prediction": self.total_best_sum["prediction"]})
@@ -95,6 +96,9 @@ class Ensemble:
             # ]
         )
         ensemble_oof["target"] = self.target
+        if self.sample_weight is not None:
+            ensemble_oof["sample_weight"] = self.sample_weight
+
         self.oof_predictions = ensemble_oof
         return ensemble_oof
 
@@ -128,7 +132,10 @@ class Ensemble:
                     self.target_columns
                 ]  # it will be needed for computing advance model statistics
 
-        return oofs, self.target
+            if self.sample_weight is None and "sample_weight" in oof.columns:
+                self.sample_weight = oof["sample_weight"]
+
+        return oofs, self.target, self.sample_weight
 
     def get_additional_metrics(self):
         if self._additional_metrics is None:
@@ -150,17 +157,19 @@ class Ensemble:
                 )
                 oof_preds["label"] = oof_preds["label"].map(labels)
 
+            sample_weight = None
+            if "sample_weight" in oof_predictions.columns:
+                sample_weight = oof_predictions["sample_weight"]
+
             self._additional_metrics = AdditionalMetrics.compute(
-                oof_predictions[target_cols],
-                oof_preds,  # oof_predictions[prediction_cols],
-                self._ml_task,
+                oof_predictions[target_cols], oof_preds, sample_weight, self._ml_task
             )
             if self._ml_task == BINARY_CLASSIFICATION:
                 self._threshold = float(self._additional_metrics["threshold"])
 
         return self._additional_metrics
 
-    def fit(self, oofs, y):
+    def fit(self, oofs, y, sample_weight=None):
         logger.debug("Ensemble.fit")
         start_time = time.time()
         selected_algs_cnt = 0  # number of selected algorithms
@@ -173,7 +182,7 @@ class Ensemble:
             # try to add some algorithm to the best_sum to minimize metric
             for model_name in oofs.keys():
                 y_ens = self._get_mean(oofs[model_name], best_sum, j + 1)
-                score = self.metric(y, y_ens)
+                score = self.metric(y, y_ens, sample_weight)
 
                 if self.metric.improvement(previous=min_score, current=score):
                     min_score = score
@@ -209,7 +218,8 @@ class Ensemble:
             ]
             logger.debug(f"{model_name} {self.best_algs.count(model_name)}")
 
-        self.get_additional_metrics()
+        self._additional_metrics = self.get_additional_metrics()
+
         self.train_time = time.time() - start_time
 
     def predict(self, X, X_stacked=None):
@@ -323,6 +333,7 @@ class Ensemble:
 
         LearningCurves.plot_for_ensemble(self._scores, self.metric.name, model_path)
 
+        # call additional metics just to be sure they are computed
         self._additional_metrics = self.get_additional_metrics()
 
         AdditionalMetrics.save(
@@ -338,7 +349,8 @@ class Ensemble:
             select_models_desc += [
                 {"model": selected["model"]._name, "repeat": selected["repeat"]}
             ]
-        desc = f"# Summary of {self.get_name()}\n"
+        desc = f"# Summary of {self.get_name()}\n\n"
+        desc += "[<< Go back](../README.md)\n\n"
         desc += "\n## Ensemble structure\n"
         selected = pd.DataFrame(select_models_desc)
         desc += tabulate(selected.values, ["Model", "Weight"], tablefmt="pipe")
